--[[	PhantomPlates PowerBars Module
	by SDPhantom
	https://www.wowinterface.com/forums/member.php?u=34145
	https://www.curseforge.com/members/sdphantomgamer/projects	]]
--------------------------------------------------------------------------

--------------------------
--[[	Namespace	]]
--------------------------
local AddOn=select(2,...);

----------------------------------
--[[	Import From Utilities	]]
----------------------------------
local AddOn_Utilities_UnitPowerBarColor=AddOn.Utilities.UnitPowerBarColor;

------------------------------------------
--[[	HealthBar Color Adjustments	]]
------------------------------------------
--	Lock friendly health bars to green, otherwise mana bars are hard to see
--DefaultCompactNamePlateFriendlyFrameOptions.colorHealthBySelection=false;

------------------
--[[	Hooks	]]
------------------
local PowerBars={};

AddOn.HookNamePlateUFCreated(function(_,base)
	local unitframe=base.UnitFrame;
	if not PowerBars[unitframe] then
		local bar=unitframe.healthBar:CreateTexture(nil,"BORDER",nil,0);
		bar:SetTexture("Interface/TargetingFrame/UI-TargetingFrame-BarFill");
		bar:SetDrawLayer("BORDER",1);
		bar:SetPoint("TOPLEFT",unitframe.healthBar,"LEFT");
		bar:SetPoint("BOTTOM");
		bar:SetWidth(unitframe.healthBar:GetWidth());

		if not AddOn.Options.PowerBars_Enabled then bar:Hide(); end
		PowerBars[unitframe]=bar;
	end
end);

--	These functions are shared with other UnitFrames
local function UpdatePower(self)
	local bar,displayedunit=PowerBars[self],self.displayedUnit;
	if not (bar and AddOn.Options.PowerBars_Enabled) then return; end

	local powertype=UnitPowerType(displayedunit);
	local cur,max=UnitPower(displayedunit,powertype),UnitPowerMax(displayedunit,powertype);
	if max>0 then
		local percent=cur/max;
		local width=percent*self.healthBar:GetWidth();
		if width>=1 then
			bar:Show();
			bar:SetWidth(width);
			bar:SetTexCoord(0,percent,0,1);
			return;
		end
	end
	bar:Hide();
end

local function UpdatePowerColor(self)
	if not (PowerBars[self] and AddOn.Options.PowerBars_Enabled) then return; end
	PowerBars[self]:SetVertexColor(AddOn_Utilities_UnitPowerBarColor(self.displayedUnit));
end

--	:OnAdded() calls CompactUnitFrame_UpdateAll(), which calls these anyway
hooksecurefunc("CompactUnitFrame_UpdateMaxPower",UpdatePower);
hooksecurefunc("CompactUnitFrame_UpdatePower",UpdatePower);
hooksecurefunc("CompactUnitFrame_UpdatePowerColor",UpdatePowerColor);

--	Attempt to lock health bar colors
hooksecurefunc("CompactUnitFrame_UpdateHealthColor",function(self)
	if not PowerBars[self] then return; end
	local r,g,b; if UnitPlayerControlled(self.unit) then
		if UnitCanAttack("player",self.unit) then--	Separate by PvP-enabled enemy players
			local faction=UnitFactionGroup("player");
			if UnitIsPVPFreeForAll("player") or (faction and faction~="Neutral" and UnitIsPVP("player")) then
				r,g,b=1,0,0;--	Hostile if we're PvP-flagged too
			else r,g,b=1,1,0; end--	Neutral; unit is PvP-flagged, but we aren't
		else r,g,b=0,1,0; end--	Default to friendly
	else r,g,b=UnitSelectionColor(self.unit); end--	Use selection color for non-players
	self.healthBar:SetStatusBarColor(r,g,b);
end);

--------------------------
--[[	Options Setup	]]
--------------------------
local OptionsLayoutSection=1;--AddOn.GetPreferredOptionsLayoutSection();
AddOn.AddOptionsLayoutPadding(OptionsLayoutSection);

AddOn.RegisterOptionsControl("PowerBars_Enabled",true
	,"OnClick","SetChecked","GetChecked"
	,function(_,_,val)--	Callback
		for unitframe,bar in pairs(PowerBars) do
			bar:SetShown(val);
			if unitframe.displayedUnit then
				UpdatePower(unitframe);
				UpdatePowerColor(unitframe);
			end
		end
	end
	,function(panel)--	Initialization
		local button=CreateFrame("CheckButton",nil,panel,"UICheckButtonTemplate");
		button:SetPoint("TOPLEFT",panel,AddOn.GetNextOptionsLayoutSectionPosition(OptionsLayoutSection,0,24));
		button.text:SetText(AddOn.Localization.OptionsSetting_PowerBars_Enabled);
		return button;
	end
);
